% "teedesign" tunes a high-pass tee-match
% having a lossy inductor, specified by L and Q,
% to a reactive load specified by series resistance
% and series reactance (negative for capacitive)
% Capacitors are assumed lossless.
% requires functions "getval", "yfun" 
% "teefit.m" and "teemodel2.m"
%
% Algorithm: Uses binary bisection to find R0 from specified L,
% then designs lossless transmatch with this R0,
% then "fminsearch" to find adjusted lossy values of C1, C2.
% Then solves network using matrix mesh analysis.
%
% Parameter input: Default values are shown in brackets
% Minimum L value (equal reactance) shown before L entry
% Either default or supplied value are accepted by <enter>.
% 
% Output: Shows component values for a lossless inductor,
% then adjusted values for lossy inductor.
% Also shows power lost, peak and rms component voltages.
%  
% Input impedance of tee assumed as 50 ohm resistive.
% 
% geb 28 October 2010

clear;

global zind R2 X2 x1sav x2sav Y;	
global Y1 Y2 YL;
more off;		% disable paging
plotflag = 0;	% 0 for no fminsearch plot, 1 to plot
R1 = 50; 		% default input resistance
printf('\n\n---------------------------------\n');
R2 = getval('Load resistance?  (50)  ',50);
X2 = getval('Load reactance?   (zero)  ',0);
f = getval('Design frequency, MHz?  (3.5) ', 3.5);
Q = getval('Inductor Q?  (100)  ',100);
printflag = getval('Expanded results?  (1 for yes, return for no)  ',0);

Xlmin = sqrt(R1 * R2);			% Find minimum inductance
Lmin = Xlmin/(2 * pi * f);		% f was in MHz, Lmin in uH

% choose default inductance 1% > than equal reactance.
% may return C2 inductive if load highly capacitive

printf('\nMinimum inductance is  %4.2f uH\n', Lmin * 1.01);
L = getval('Inductance, uH?  (Minimum value)   ', Lmin * 1.01);

Y1 = 1/R1;					% setup for inversion for R0
Y2 = 1/R2;
XL = 2 * pi * f * L;	    % inductor reactance
YL = 1/XL;
% x1 = 1/(50 + R2);			% lower search limit
x1 = 1/(50 + R2);			% lower search limit
x2 = 1/15000;				% Upper search limit

tol = 1e-7; OKflag = 0;		% setup and search
while (abs(yfun(x1) - yfun(x2)) > tol)
	xmid = (x2 + x1)/2;
	if (yfun(x1) * yfun(xmid) < 0) x2 = xmid;
	    else x1 = xmid; 
		OKflag = 1;			% coming here means convergence
	endif;
endwhile
if OKflag == 0
	printf('\n\nOops! Convergence failed!  Increase inductance!\n');
endif	

R0 = 2/(x1 + x2);			% characteristic impedance
	
x1 = -sqrt(R1 * (R0 - R1));		% find capacitor reactance magnitudes
x2 = -sqrt(R2 * (R0 - R2));		% ignoring load reactance
yp = -x1/(R1*R0) - x2/(R0*R2); xp = 1/yp;	% Inductor reactance
zind = xp/Q + j * xp;	        % inductor impedance
[zin1, Av] = teemodel2(x1,x2);  % Find unadjusted input impedance
Plest = (100/Q)*(sqrt(R0/R1 - 1) + sqrt(R0/R2 - 1)); % Estimated loss

p(1) = x1; p(2) = x2;	       % starting capacitor reactances
[x1sav] = x1; [x2sav] = x2;	   % ignoring load reactance	

% Use This call for simple output
% Return Xc1, Xc2 as e(1), e(2)
% ignore load reactance, adjust later.
   e = fminsearch(@teefit,p);     % Return Xc1, Xc2 as e(1), e(2).
  
 % Use this call for expanded output.  
  % e = fminsearch(@teefit,p,[1,1e-6,0,0,0,0]);    				

if printflag != 0
  printf('Unadjusted Zin       = %6.2f + %6.2fj \n', real(zin1), imag(zin1));
  printf('Estimated power loss = %6.2f  Watt\n',Plest);
end;  

Xc2a = real(x2) - X2;		   % adjust Xc2 for load reactance
if printflag != 0
fprintf('\nLossless design values Xc1 = %6.2f,   Xc2 = %6.2f   XL = %6.2f', x1, Xc2a, xp);
endif

Xc2b = e(2)-X2;		% adjust Xc2 for load reactance
if printflag != 0
  fprintf('\nAdjusted lossy values  Xc1 = %6.2f,   Xc2 = %6.2f \n', e(1), Xc2b);
endif;
[zin, Av] = teemodel2(e(1), e(2));

fprintf('Adjusted Zin    = %6.2f + %6.2fj \n', real(zin), imag(zin));

fminflag = 0;
if abs((zin) - 50 > 0.2)	% check for fmins convergence
	fminflag = 1;
endif

% fprintf('Gain magnitude  = %6.2f \n', abs(Av)/2);
Vload =  70.7107 * abs(Av);
fprintf('Load voltage    = %6.2f Volt rms \n',Vload);
Pload = Vload^2/R2;
fprintf('Power in load   = %6.2f Watt  (100 Watt in) \n',Pload);
fprintf('True Power loss = %6.2f Watt\n\n', 100-Pload);

% Find component values required

fprintf('\nFor matching at %4.2f MHz,',f);
w = 2 * pi *f * 1e6;	% operating frequency
C1 = -1e12/(w * e(1)); C2 = -1e12/(w * Xc2b);
L = xp * 1e6/w;
fprintf('\n    C1 = %6.2f pF  (Lossless %6.2f pf)',C1,1e12/(-x1*w));
fprintf('\n    C2 = %6.2f pF  (Lossless %6.2f pf)',C2,1e12/((-x2+X2)*w));
fprintf('\n     L = %6.2f uH\n',L);

if Xc2b > 0 	% warn if Xc2 is positive!
   fprintf('\nWarning! C2 negative!  Increase inductance value\n');
   end

% Find rms voltages across reactances from Y matrix

Vc1 = -70.711 * abs(Y(1,1)) * e(1);
Vc2 = -70.711 * abs(Y(2,1)) * e(2);
VL = 70.711 * abs((Y(2,1) - Y(1,1)) *zind);

if printflag != 0
 fprintf('\n Vc1             = %3.2f Volt rms',Vc1);
 fprintf('\n Vc2             = %3.2f Volt rms',Vc2);
 fprintf('\n Vinductor       = %3.2f Volt rms \n',VL);
endif; 

fprintf('\n Peak Vc1        = %3.2f Volt',Vc1*sqrt(2));
fprintf('\n Peak Vc2        = %3.2f Volt',Vc2*sqrt(2));
fprintf('\n Peak Vinductor  = %3.2f Volt',VL*sqrt(2));
if printflag != 0
 fprintf('\n\n %g calls to fitting function were required\n',length(x1sav));
endif;
fprintf('\n');

if OKflag == 0
	printf('\n\nOops!  R0 convergence failed!  Increase inductance!\n');
endif

if fminflag ==1
    fprintf('\n But adjusted Zin = %6.2f + %6.2fj \n', real(zin), imag(zin)); 
	printf('fminsearch failed convergence!  Increase L!\n');
	
endif		

% plot iterated values of Xc1 and Xc2
 if plotflag == 1

  plot(x1sav); hold on; plot(x1sav,'o');
  plot(x2sav - X2,'r'); plot(x2sav - X2,'or'); grid;
  xlabel('Iteration number');
  ylabel('Iterated reactance values');
  legend('Xs1', 'Xs1', 'Xs2', 'Xs2', 'boxon');
  hold off; figure(gcf);
endif